<?PHP
	// returns a list of valid domains from the conf file
	function getconffiledomains()
	{
		// read the configuration file into an array
		$conffile = file('include/ismail.conf');

		$section = '';
		$sort = false;
		$domains = array();
		$end = count($conffile);
		for($i=0;$i<$end;$i++) {
			if(preg_match('/^\s*[a-zA-Z0-9.-]+:\s*$/', $conffile[$i])) {
				$domain = substr(trim($conffile[$i]), 0, -1);
				if(strtolower($domain) != 'default') {
					$domains[] = $domain;
					$section = 'not default';
				}
				else {
					$section = 'default';
				}
			}
			elseif($section == 'default' and preg_match('/^\s*sortdomains\s*=\s*([a-z0-9]+)\s*$/i', $conffile[$i], $matches)) {
				if(strtolower($matches[1]) == 'true' or $matches[1] == '1')
					$sort = true;
			}
		}

		if($sort == true)
			sort($domains);

		return $domains;
	}


	// parses a string (possible with replacement variables) and returns
	// it without quotes and with all replacements made
	function parseconffilestring($str, $replacements = "")
	{
		// strip quotes from beginning and end
		if($str[0] == '"' and $str[strlen($str)-1] == '"') {
			$str = substr($str,1,-1);
			$str = str_replace("\\\"", '"', $str);
			$str = str_replace("\\\\", "\\", $str);
		}

		if($replacements != "") {
			while(list($key, $value) = each($replacements)) {
				$str = str_replace($key, $value, $str);
			}
		}

		return $str;
	}


	// takes a string from the theme file that *should* be an option string and
	// attempts to parse it as such.  Returns false if it cannot parse it properly.
	function getthemefileoption($str)
	{
		// debugging assertions - this function should never be passed
		// whitespace or comments or untrimmed strings.
		assert('$str = trim($str)');
		assert('$str != \'\'');
		assert('$str[0] != "#"');

		// split the string into option and setting
		$split = explode('=', $str);
		if(count($split) < 2) {
			return false;
		}
		$option = trim($split[0]);
		unset($split[0]);

		$setting = trim(implode('=', $split));

		if(strpos($option, ' ') !== false) {
			return false;
		}

		// setup a return array and a default tuple containing the passed in option.
		// When defining multiple tuples in a setting DO NOT add the final tuple to
		// $returnsettings because that will be done automatically at the end of the
		// switch.
		$returnsettings = array();
		$tuple = array();
		$tuple['option'] = $option;
		$tuple['setting'] = $setting;
		if($tuple['setting'][0] == '"')
			$tuple['setting'] = parseconffilestring($tuple['setting']);
		switch($option) {
			case 'maillist-highlight-color':
				break;
			case 'link-color':
				break;
			case 'link-hover-color':
				break;
			case 'link-visited-color':
				break;
			case 'addressbookframe-link-color':
				break;
			case 'addressbookframe-link-visited-color':
				break;
			case 'addressbookframe-link-hover-color':
				break;
			case 'addressbook-link-color':
				break;
			case 'addressbook-link-visited-color':
				break;
			case 'addressbook-link-hover-color':
				break;
			case 'mail-link-color':
				break;
			case 'mail-link-visited-color':
				break;
			case 'mail-link-hover-color':
				break;
			case 'folderlist-link-color':
				break;
			case 'folderlist-link-hover-color':
				break;
			case 'topbar-height':
				break;
			case 'topbar-button-height':
				break;
			case 'topbar-button-width':
				break;
			case 'topbar-separator-height':
				break;
			case 'topbar-separator-width':
				break;
			default:
				return false;
		}

		return $tuple;
	}


	// takes a string from the conffile that *should* be an option setting string and
	// attempts to parse it as such.  Returns false if it cannot parse it properly.
	function getconffileoption($str, $currentsettings, $username, $domain)
	{
		// debugging assertions - this function should never be passed
		// whitespace or comments or untrimmed strings.
		assert('$str = trim($str)');
		assert('$str != ""');
		assert('$str[0] != "#"');

		// split the string into option and setting
		$split = explode('=', $str);
		if(count($split) < 2) {
			return false;
		}
		$option = trim($split[0]);
		unset($split[0]);

		$setting = trim(implode('=', $split));

		if(strpos($option, ' ') !== false) {
			return false;
		}

		// setup a return array and a default tuple containing the passed in option.
		// When defining multiple tuples in a setting DO NOT add the final tuple to
		// $returnsettings because that will be done automatically at the end of the
		// switch.
		$returnsettings = array();
		$tuple = array();
		$tuple['option'] = $option;
		$tuple['setting'] = $setting;
		switch($option) {
			case 'server':
				if($tuple['setting'][0] == '"')
					$tuple['setting'] = parseconffilestring($tuple['setting']);
				break;
			case 'port':
				if($tuple['setting'][0] == '"')
					$tuple['setting'] = parseconffilestring($tuple['setting']);
				$tuple['setting'] = (int)$tuple['setting'];
				if($tuple['setting'] < 1 or $tuple['setting'] > 65534)
					return false;
				break;
			case 'tls':
				if(strtolower($tuple['setting']) == 'true' or $tuple['setting'] == '1')
					$tuple['setting'] = true;
				elseif(strtolower($tuple['setting']) == 'false' or $tuple['setting'] == '0')
					$tuple['setting'] = false;
				else
					return false;
				break;
			case 'usernameprefix':
				if ((strlen($tuple['setting']) > 0) && ($tuple['setting'][0] == '"'))
					$tuple['setting'] = parseconffilestring($tuple['setting']);
				break;
			case 'usernamesuffix':
				if($tuple['setting'][0] == '"')
					$tuple['setting'] = parseconffilestring($tuple['setting']);
				elseif($tuple['setting'] == 'domain')
					$tuple['setting'] = '@' . $domain;
				break;
			case 'rootfolderpath':
				if($tuple['setting'][0] == '"')
					$tuple['setting'] = parseconffilestring($tuple['setting']);
				break;
			case 'folderseparator':
				if($tuple['setting'][0] == '"')
					$tuple['setting'] = parseconffilestring($tuple['setting']);
				break;
			case 'email':
				$replacements = array();
				$replacements['$username'] = $username;
				$replacements['$domain'] = $domain;
				$replacements['$usernameprefix'] = $currentsettings['usernameprefix'];
				$replacements['$usernamesuffix'] = $currentsettings['usernamesuffix'];
				$tuple['setting'] = parseconffilestring($tuple['setting'], $replacements);
				break;
			case 'sentitemsfolder':
				if($tuple['setting'][0] == '"')
					$tuple['setting'] = parseconffilestring($tuple['setting']);
				break;
			case 'serversorting':
				if(strtolower($tuple['setting']) == 'true' or $tuple['setting'] == '1')
					$tuple['setting'] = true;
				elseif(strtolower($tuple['setting']) == 'false' or $tuple['setting'] == '0')
					$tuple['setting'] = false;
				else
					return false;
				break;
			case 'mailprogram':
				if($tuple['setting'][0] == '"')
					$tuple['setting'] = parseconffilestring($tuple['setting']);
				$replacements = array();
				$replacements['$username'] = $username;
				$replacements['$domain'] = $domain;
				$replacements['$usernameprefix'] = $currentsettings['usernameprefix'];
				$replacements['$usernamesuffix'] = $currentsettings['usernamesuffix'];
				$replacements['$emailaddress'] = $currentsettings['email'];
				$tuple['setting'] = parseconffilestring($tuple['setting'], $replacements);
				break;
			case 'extensionblacklist':
				if($tuple['setting'][0] == '"')
					$tuple['setting'] = parseconffilestring($tuple['setting']);
				if(trim($tuple['setting']) == "")
					$tuple['setting'] = array();
				else
					$tuple['setting'] = explode(':', $tuple['setting']);
				break;
			case 'addressbookstorage':
				if($tuple['setting'][0] == '"')
					$tuple['setting'] = parseconffilestring($tuple['setting']);
				$tuple['setting'] = strtolower(trim($tuple['setting']));
				if($tuple['setting'] == 'xml') {
					$tuple['setting'] = 'include/addressbookxml.class';
				}
				elseif($tuple['setting'] == 'encryptedxml') {
					$tuple['setting'] = 'include/addressbookxml.class';
					$returnsettings[] = $tuple;
					$tuple = array();
					$tuple['option'] = 'encryptaddressbooks';
					$tuple['setting'] = 1;
				}
				elseif($tuple['setting'] == 'postgresql') {
					$tuple['setting'] = 'include/addressbookpgsql.class';
				}
				else {
					return false;
				}
				break;
			case 'dictenable':
				if(strtolower($tuple['setting']) == 'true' or $tuple['setting'] == '1')
					$tuple['setting'] = true;
				elseif(strtolower($tuple['setting']) == 'false' or $tuple['setting'] == '0')
					$tuple['setting'] = false;
				else
					return false;
				break;
			case 'dictcommand':
				if($tuple['setting'][0] == '"')
					$tuple['setting'] = parseconffilestring($tuple['setting']);
				break;
			case 'dictargs':
				if($tuple['setting'][0] == '"')
					$tuple['setting'] = parseconffilestring($tuple['setting']);
				break;
			case 'spellenable':
				if(strtolower($tuple['setting']) == 'true' or $tuple['setting'] == '1')
					$tuple['setting'] = true;
				elseif(strtolower($tuple['setting']) == 'false' or $tuple['setting'] == '0')
					$tuple['setting'] = false;
				else
					return false;
				break;
			case 'spellcommand':
				if($tuple['setting'][0] == '"')
					$tuple['setting'] = parseconffilestring($tuple['setting']);
				break;
			case 'spellargs':
				if($tuple['setting'][0] == '"')
					$tuple['setting'] = parseconffilestring($tuple['setting']);
				break;
			case 'sortdomains':
				if(strtolower($tuple['setting']) == 'true' or $tuple['setting'] == '1')
					$tuple['setting'] = true;
				elseif(strtolower($tuple['setting']) == 'false' or $tuple['setting'] == '0')
					$tuple['setting'] = false;
				elseif(strtolower($tuple['setting']) == 'desc' or strtolower($tuple['setting']) == 'descending')
					$tuple['setting'] = 'desc';
				else
					return false;
				break;
			case 'imapflavor':
				if(strtolower($tuple['setting']) == 'courier-imap') {
					$tuple['setting'] = 'courier-imap';
					$returnsettings[] = $tuple;
					$tuple = array();
					$tuple['option'] = 'folderseparator';
					$tuple['setting'] = '.';
					$returnsettings[] = $tuple;
					$tuple = array();
					$tuple['option'] = 'rootfolderpath';
					$tuple['setting'] = 'INBOX';
					$returnsettings[] = $tuple;
					$tuple = array();
					$tuple['option'] = 'sentitemsfolder';
					$tuple['setting'] = 'Sent Items';
					$returnsettings[] = $tuple;
					$tuple = array();
					$tuple['option'] = 'serversorting';
					$tuple['setting'] = 1;
				}
				elseif(strtolower($tuple['setting']) == 'exchange') {
					$tuple['setting'] = 'exchange';
					$returnsettings[] = $tuple;
					$tuple = array();
					$tuple['option'] = 'folderseparator';
					$tuple['setting'] = '/';
					$returnsettings[] = $tuple;
					$tuple = array();
					$tuple['option'] = 'rootfolderpath';
					$tuple['setting'] = '';
					$returnsettings[] = $tuple;
					$tuple = array();
					$tuple['option'] = 'sentitemsfolder';
					$tuple['setting'] = 'Sent';
					$returnsettings[] = $tuple;
					$tuple = array();
					$tuple['option'] = 'serversorting';
					$tuple['setting'] = 0;
				}
				elseif(strtolower($tuple['setting']) == 'uwimap') {
					$tuple['setting'] = 'uwimap';
					$returnsettings[] = $tuple;
					$tuple = array();
					$tuple['option'] = 'folderseparator';
					$tuple['setting'] = '/';
					$returnsettings[] = $tuple;
					$tuple = array();
					$tuple['option'] = 'rootfolderpath';
					$tuple['setting'] = 'mail';
					$returnsettings[] = $tuple;
					$tuple = array();
					$tuple['option'] = 'sentitemsfolder';
					$tuple['setting'] = 'Sent Items';
					$returnsettings[] = $tuple;
					$tuple = array();
					$tuple['option'] = 'serversorting';
					$tuple['setting'] = 1;
				}
				break;
			case 'themedir':
				if($tuple['setting'][0] == '"')
					$tuple['setting'] = parseconffilestring($tuple['setting']);
				break;
			default:
				return false;
		}

		// all went well, add the dangling tuple to the returnsettings and send them back to the caller
		$returnsettings[] = $tuple;
		return $returnsettings;
	}



	// reads in the configuration file and returns a hash of settings from it
	// returns either an array/hash of the settings or an array of length one
	// that contains only the key 'error' and a string describing the error
	// encountered.
	function readconffile($username, $domain="")
	{
		// set defaults for all necessary variables
		$settings = array();
		$settings['domain'] = $domain;
		$settings['imapflavor'] = 'courier-imap';
		$settings['server'] = '127.0.0.1';
		$settings['port'] = 143;
		$settings['tls'] = false;
		$settings['usernameprefix'] = '';
		$settings['usernamesuffix'] = '@' . $domain;
		$settings["folderseparator"] = ".";
		$settings['rootfolderpath'] = "INBOX";
		$settings['email'] = $username . '@' . $domain;
		$settings['serversorting'] = 1;
		$settings['encryptaddressbooks'] = 0;
		$settings['mailprogram'] = '"/usr/libexec/sendmail/sendmail -oi -t"';
		$settings['extensionblacklist'] = array('php', 'php4', 'phtml', 'phps', 'pl', 'asp', 'rb', 'py', 'pyc');
		$settings['addressbookstorage'] = 'include/addressbookxml.class';
		$settings['dictenable'] = 0;
		$settings['spellenable'] = 0;
		$settings['themedir'] = "default";

		// read the configuration file into an array
		$conffile = file('include/ismail.conf');

		// go through the array line by line overriding defaults
		$section = "";
		$end = count($conffile);

		# the default section must come first - read through until we find it
		for($i=0;$i<$end;$i++) {
			$str = trim($conffile[$i]);

			// ignore whitespace and comments
			if($str == '')
				continue;
			if($str[0] == '#')
				continue;

			// check for the start of the default section
			if($str == 'default:') {
				$section = "default";
				break;
			}

			$error = array();
			$error['error'] = 'The first section in the configuration file must be default:';
			return $error;
		}

		// now read through the default section setting values until we hit another section
		for($i=$i+1;$i<$end;$i++) {
			$str = trim($conffile[$i]);

			// ignore whitespace and comments
                        if($str == '')
                                continue;
                        if($str[0] == '#')
                                continue;

			// check for a new section
			if(preg_match("/^[a-zA-Z0-9.-]+:$/", $str) == 1) {
				$section = substr($str, 0, -1);
				break;
			}

			// try to handle everything else as an option string
			$options = getconffileoption($str, $settings, $username, $domain);
			if($options === false) {
				$error = array();
				$error['error'] = 'Unknown or malformed option on line #' . ($i+1);
				return $error;
			}
			else {
				$oend = count($options);
				for($o=0;$o<$oend;$o++) {
					$settings[$options[$o]['option']] = $options[$o]['setting'];
				}
			}
		}


		// now read through the conffile until we find the domain section we are logging in to
		if($section != $domain) {
			for($i=$i+1;$i<$end;$i++) {
				$str = trim($conffile[$i]);

				// check for a new section
				if(preg_match("/^[a-zA-Z0-9.-]+:$/", $str) == 1) {
					$section = substr($str, 0, -1);
					if($section == $domain)
						break;
				}
			}
		}

		if($section != $domain) {
			$error = array();
			$error['error'] = "Unable to find domain '" . $domain . "' in configuration.";
			return $error;
		}

		for($i=$i+1;$i<$end;$i++) {
			$str = trim($conffile[$i]);

			// ignore whitespace and comments
                        if($str == '')
                                continue;
                        if($str[0] == '#')
                                continue;

			// check for a new section
			if(preg_match("/^[a-zA-Z0-9.-]+:$/", $str) == 1) {
				$section = substr($str, 0, -1);
				break;
			}

			// try to handle everything else as an option string
			$options = getconffileoption($str, $settings, $username, $domain);
			if($options === false) {
				$error = array();
				$error['error'] = 'Unknown or malformed option on line #' . ($i+1);
				return $error;
			}
			else {
				$oend = count($options);
				for($o=0;$o<$oend;$o++) {
					$settings[$options[$o]['option']] = $options[$o]['setting'];
				}
			}
		}

		return $settings;
	}


// Reads in a theme file and returns a full hash of theme settings.
// On error it returns a hash with the single entry 'error' and a
// string describing the error encountered.
function readthemefile($themedir) {
	// set defaults for all necessary variables
	$settings = array();
	$settings['maillist-highlight-color'] = '#78FFFF';
	$settings['link-color'] = '#0000FF';
	$settings['folderlist-link-color'] = '#000000';
	$settings['folderlist-link-hover-color'] = '#0000FF';
	$settings['topbar-height'] = '50px';
	$settings['topbar-separator-width'] = '25px';

	// read the theme file into an array
	$conffile = @file('graphics/themes/' . $themedir . '/istheme.conf');

	// if the theme files can't be found - return false
	if($conffile === false) {
		$error = array();
		$error['error'] = "Unable to open istheme.conf for theme " . $themedir;
		return $error;
	}

	// go through the array line by line overriding defaults
	$end = count($conffile);

	for($i=0;$i<$end;$i++) {
		$str = trim($conffile[$i]);

		// ignore whitespace and comments
		if($str == '')
			continue;
		if($str[0] == '#')
			continue;

		// try to handle everything else as an option string
		$option = getthemefileoption($str);
		if($option === false) {
			$error = array();
			$error['error'] = 'Unknown or malformed theme option on line #' . ($i+1);
			return $error;
		}
		else {
			$settings[$option['option']] = $option['setting'];
		}
	}

	// set things that cascade from other things unless they have been overridden
	if(!isset($settings['topbar-button-height']))
		$settings['topbar-button-height'] = $settings['topbar-height'];
	if(!isset($settings['topbar-button-width']))
		$settings['topbar-button-width'] = $settings['topbar-height'];

	if(!isset($settings['topbar-separator-height']))
		$settings['topbar-separator-height'] = $settings['topbar-height'];

	if(!isset($settings['link-visited-color']))
		$settings['link-visited-color'] = $settings['link-color'];
	if(!isset($settings['link-hover-color']))
		$settings['link-hover-color'] = $settings['link-color'];
	if(!isset($settings['addressbookframe-link-color']))
		$settings['addressbookframe-link-color'] = $settings['link-color'];
	if(!isset($settings['addressbookframe-link-visited-color']))
		$settings['addressbookframe-link-visited-color'] = $settings['link-visited-color'];
	if(!isset($settings['addressbookframe-link-hover-color']))
		$settings['addressbookframe-link-hover-color'] = $settings['link-hover-color'];

	if(!isset($settings['addressbook-link-color']))
		$settings['addressbook-link-color'] = $settings['addressbookframe-link-color'];
	if(!isset($settings['addressbook-link-visited-color']))
		$settings['addressbook-link-visited-color'] = $settings['addressbookframe-link-visited-color'];
	if(!isset($settings['addressbook-link-hover-color']))
		$settings['addressbook-link-hover-color'] = $settings['addressbookframe-link-hover-color'];

	if(!isset($settings['mail-link-color']))
		$settings['mail-link-color'] = $settings['link-color'];
	if(!isset($settings['mail-link-visited-color']))
		$settings['mail-link-visited-color'] = $settings['link-visited-color'];
	if(!isset($settings['mail-link-hover-color']))
		$settings['mail-link-hover-color'] = $settings['link-hover-color'];

	return $settings;
}

?>
